import os
import logging
import importlib

from twisted.internet import defer

from torba.manager import WalletManager

from orchstr8.services import TorbaService, ElectrumXService, BlockchainService


def get_ledger_from_environment():
    if 'LEDGER' not in os.environ:
        raise ValueError('Environment variable LEDGER must point to a torba based ledger module.')
    module_name = os.environ['LEDGER'].split('-')[-1]  # tox support
    return importlib.import_module(module_name)


def get_electrumx_from_ledger(ledger_module):
    electrumx_path, regtest_class_name = ledger_module.__electrumx__.rsplit('.', 1)
    electrumx_module = importlib.import_module(electrumx_path)
    return getattr(electrumx_module, regtest_class_name)


def set_logging(ledger_module, verbosity):
    level = logging.DEBUG if verbosity else logging.WARNING
    defer.setDebugging(verbosity)
    logging.getLogger('torba').setLevel(level)
    logging.getLogger('asyncio').setLevel(level)
    logging.getLogger('twisted').setLevel(level)
    logging.getLogger('electrumx').setLevel(level)
    logging.getLogger(ledger_module.__name__).setLevel(level)
    logging.getLogger(ledger_module.__electrumx__.split('.')[0]).setLevel(level)


class ServerStack(object):

    def __init__(self, ledger_module=None, verbose=False):
        if ledger_module is None:
            ledger_module = get_ledger_from_environment()
        set_logging(ledger_module, verbose)
        self.blockchain = BlockchainService(
            ledger_module.__node_url__,
            os.path.join(ledger_module.__node_bin__, ledger_module.__node_daemon__),
            os.path.join(ledger_module.__node_bin__, ledger_module.__node_cli__),
            verbose
        )
        self.electrumx = ElectrumXService(get_electrumx_from_ledger(ledger_module), verbose)
        self.ledger_module = ledger_module

    async def start(self):
        await self.blockchain.start()
        await self.blockchain.generate(200)
        await self.electrumx.start()

    async def stop(self, cleanup=True):
        try:
            await self.electrumx.stop(cleanup=cleanup)
        except Exception as e:
            print(e)

        try:
            await self.blockchain.stop(cleanup=cleanup)
        except Exception as e:
            print(e)


class FullStack(ServerStack):

    def __init__(self, wallet_manager_class=WalletManager, ledger_module=None, verbose=False):
        super().__init__(ledger_module, verbose)
        self.wallet = TorbaService(wallet_manager_class, self.ledger_module.RegTestLedger, verbose)

    async def start(self):
        await super().start()
        await self.wallet.start()

    async def stop(self, cleanup=True):
        try:
            await self.wallet.stop(cleanup=cleanup)
        except Exception as e:
            print(e)

        await super().stop(cleanup)



import os
import shutil
import zipfile
from random import Random

from twisted.internet import defer, task

from lbrynet.core.utils import generate_id
from lbryschema.claim import ClaimDict
from orchstr8.services import Lbrycrd


class Fixture:

    def __init__(self, blocks=100, txns_per_block=100, seed=2015, start_blocks=110):
        self.blocks = blocks
        self.txns_per_block = txns_per_block
        self.start_blocks = start_blocks
        self.random = Random(seed)
        self.lbrycrd = Lbrycrd(verbose=True)

    @defer.inlineCallbacks
    def start(self):
        self.lbrycrd.setup()
        yield self.lbrycrd.start()
        yield self.lbrycrd.generate(self.start_blocks)

    @defer.inlineCallbacks
    def generate_transactions(self):
        for block in range(self.blocks):
            for txn in range(self.txns_per_block):
                name = 'block{}txn{}'.format(block, txn)
                amount = self.random.randrange(1, 5)/1000.0
                yield self.lbrycrd.claimname('@'+name, self._claim().serialized.encode('hex'), amount)
            yield self.lbrycrd.generate(1)

    def stop(self):
        return self.lbrycrd.stop(cleanup=False)

    def save(self):
        if not os.path.exists(self.data_dir):
            os.mkdir(self.data_dir)
        if os.path.exists(self.blockchain_zip_filename):
            os.remove(self.blockchain_zip_filename)
        shutil.make_archive(
            self.blockchain_base_filename, "zip", self.lbrycrd.data_path
        )

    def cleanup(self):
        self.lbrycrd.cleanup()

    @property
    def data_dir(self):
        return os.path.join(self.lbrycrd.project_dir, 'data')

    @property
    def blockchain_base_filename(self):
        return os.path.join(self.data_dir, 'blockchain')

    @property
    def blockchain_zip_filename(self):
        return self.blockchain_base_filename + '.zip'

    def extract(self):
        with zipfile.ZipFile(self.blockchain_zip_filename) as zf:
            zf.extractall(self.lbrycrd.data_path)

    def _claim(self):
        return ClaimDict.load_dict({
          "version": "_0_0_1",
          "claimType": "streamType",
          "stream": {
            "source": {
              "source": generate_id(self.random.getrandbits(512)).encode('hex'),
              "version": "_0_0_1",
              "contentType": "video/mp4",
              "sourceType": "lbry_sd_hash"
            },
            "version": "_0_0_1",
            "metadata": {
              "license": "LBRY Inc",
              "description": "What is LBRY? An introduction with Alex Tabarrok",
              "language": "en",
              "title": "What is LBRY?",
              "author": "Samuel Bryan",
              "version": "_0_1_0",
              "nsfw": False,
              "licenseUrl": "",
              "preview": "",
              "thumbnail": "https://s3.amazonaws.com/files.lbry.io/logo.png"
            }
          }
        })


@defer.inlineCallbacks
def generate_test_chain(_):
    fixture = Fixture(10, 10)
    yield fixture.start()
    yield fixture.generate_transactions()
    yield fixture.stop()
    fixture.save()
    fixture.cleanup()


if __name__ == "__main__":
    task.react(generate_test_chain)



import sys
import asyncio
from orchstr8.conductor import FullStack


def main():
    stack = FullStack(verbose=True)
    if sys.argv[-1] == 'download':
        stack.blockchain.ensure()
    else:
        loop = asyncio.get_event_loop()
        try:
            loop.run_until_complete(stack.start())
            print('========== Services Started ========')
            loop.run_forever()
        except KeyboardInterrupt:
            loop.run_until_complete(stack.stop())
